import { DateTime } from 'luxon';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

function __read(o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
}

var WEEKDAY = {
    ANY: '(mon|tue(s)?|wed|wedn(es)?|thu|thur(s)?|fri|sat(ur)?|sun)(day)?',
    SINGLE: {
        MONDAY: '(mon)(day)?',
        TUESDAY: '(tue(s)?)(day)?',
        WEDNESDAY: '(wed|wedn(es)?)(day)?',
        THURSDAY: '(thu(r)?(s)?)(day)?',
        FRIDAY: '(fri)(day)?',
        SATURDAY: '(sat(ur)?)(day)?',
        SUNDAY: '(sun)(day)?',
    },
    FUTURE_WORDS: 'for|next|this|current|on',
    PAST_WORDS: 'last|prev(ious)?',
};

function contains(haystack, pattern, wordBoundary) {
    if (wordBoundary === void 0) { wordBoundary = true; }
    var updatedPattern = (wordBoundary) ? "\\b" + pattern + "\\b" : pattern;
    var regex = new RegExp(updatedPattern, 'i');
    return regex.test(haystack.toLowerCase());
}

function matchPattern(haystack, pattern, wordBoundary) {
    if (wordBoundary === void 0) { wordBoundary = true; }
    var updatedPattern = (wordBoundary) ? "\\b" + pattern + "\\b" : pattern;
    var regex = new RegExp(updatedPattern, 'ig');
    return haystack.toLowerCase().match(regex);
}

function trimWhiteSpaces(text) {
    var trimmedText = text.replace(/  /g, ' ');
    return trimmedText.trim();
}

function convertWeekdayStringToNumber(weekdayString, pastWeekday) {
    var weekday = null;
    var todayInWeekday = DateTime.utc().weekday;
    Object.keys(WEEKDAY.SINGLE).forEach(function (key, index) {
        var weekdayPattern = WEEKDAY.SINGLE[key];
        if (contains(weekdayString, weekdayPattern)) {
            weekday = index + 1;
        }
    });
    if (!weekday)
        return null;
    if (weekday <= todayInWeekday)
        weekday += 7;
    if (pastWeekday)
        weekday -= 7;
    return weekday;
}

var WeekDay = (function () {
    function WeekDay() {
    }
    WeekDay.parseText = function (text) {
        var _this = this;
        var pattern = "((" + WEEKDAY.FUTURE_WORDS + "|" + WEEKDAY.PAST_WORDS + ") )?" + WEEKDAY.ANY;
        var matchForWeekdays = matchPattern(text, pattern);
        if (!matchForWeekdays)
            return null;
        return matchForWeekdays.map(function (elem) { return _this.parseWeekdayMatches(text, elem); });
    };
    WeekDay.parseWeekdayMatches = function (text, matchedWeekday) {
        var replaceMatch = text.toLowerCase().replace(matchedWeekday, '');
        return {
            text: trimWhiteSpaces(replaceMatch),
            dateTime: this.convertWeekdayMatchToDate(matchedWeekday),
            matched: trimWhiteSpaces(matchedWeekday),
        };
    };
    WeekDay.convertWeekdayMatchToDate = function (matchingText) {
        var _a = __read(matchPattern(matchingText, WEEKDAY.ANY), 1), weekdayString = _a[0];
        var pastWeekday = contains(matchingText, WEEKDAY.PAST_WORDS);
        var weekday = convertWeekdayStringToNumber(weekdayString, pastWeekday);
        var newDateTime = DateTime.utc().set({ weekday: weekday });
        if (!newDateTime.isValid)
            return null;
        return newDateTime.startOf('day').toJSDate();
    };
    return WeekDay;
}());

function parseMatches(text, pattern, dateTimeObj) {
    var regex = new RegExp(pattern, 'ig');
    var textRemain = text.replace(regex, '');
    var _a = __read(text.match(regex), 1), matched = _a[0];
    return {
        text: trimWhiteSpaces(textRemain),
        dateTime: dateTimeObj,
        matched: trimWhiteSpaces(matched),
    };
}

var day = '((3[0-1])|([1-2][0-9])|(0?[1-9]))';
var ordinalIndicators = '(st|nd|rd|th)';
var DAY = {
    ANY: "" + day + ordinalIndicators + "?",
    ORDINAL_ONLY: "" + day + ordinalIndicators,
    NUMBERS: day,
    FILLER_WORDS: 'on (the )?',
};

var Day = (function () {
    function Day() {
    }
    Day.parseText = function (text) {
        var _this = this;
        var pattern = "(" + DAY.FILLER_WORDS + ")?" + DAY.ORDINAL_ONLY;
        var matches = matchPattern(text, pattern);
        if (!matches)
            return null;
        return matches.map(function (match) {
            var dateTimeObj = _this.convertMatchToDateObj(match);
            return parseMatches(text, match, dateTimeObj);
        });
    };
    Day.convertMatchToDateObj = function (matchingText) {
        var today = DateTime.utc();
        var day = null;
        var month = DateTime.utc().month;
        if (contains(matchingText, DAY.ANY)) {
            var _a = __read(matchPattern(matchingText, DAY.ANY), 1), matchedDay = _a[0];
            day = parseInt(matchedDay, 10);
            if (day < today.day)
                month += 1;
        }
        if (!day)
            return null;
        return DateTime.utc()
            .set({ day: day, month: month })
            .startOf('day')
            .toJSDate();
    };
    return Day;
}());

var MONTH = {
    ANY: '(jan(uary)?|feb(ruary)?|mar(ch)?|apr(il)?|may|jun(e)?|jul(y)?|aug(ust)?|sep(tember)?|oct(ober)?|nov(ember)?|dec(ember)?)',
    SINGLE: {
        JANUARY: 'jan(uary)?',
        FEBRUARY: 'feb(ruary)?',
        MARCH: 'mar(ch)?',
        APRIL: 'apr(il)?',
        MAY: 'may',
        JUNE: 'jun(e)?',
        JULY: 'jul(y)?',
        AUGUST: 'aug(ust)?',
        SEPTEMBER: 'sep(tember)?',
        OCTOBER: 'oct(ober)?',
        NOVEMBER: 'nov(ember)?',
        DECEMBER: 'dec(ember)?',
    },
    NUMBERS: '(1[0-2]|0?[1-9])',
    FUTURE_WORDS: 'for|next|this|current|in',
    PAST_WORDS: 'last|prev(ious)?',
};

function monthStrToInt(matchingText) {
    var month = null;
    Object.keys(MONTH.SINGLE).forEach(function (key, index) {
        var monthPattern = MONTH.SINGLE[key];
        if (contains(matchingText, monthPattern)) {
            month = index + 1;
        }
    });
    return month;
}

var Month = (function () {
    function Month() {
    }
    Month.parseText = function (text) {
        var _this = this;
        var pattern = "((" + MONTH.FUTURE_WORDS + "|" + MONTH.PAST_WORDS + ") )?" + MONTH.ANY;
        var matches = matchPattern(text, pattern);
        if (!matches)
            return null;
        return matches.map(function (match) {
            var dateTimeObj = _this.convertMatchToDateObj(match);
            return parseMatches(text, match, dateTimeObj);
        });
    };
    Month.convertMatchToDateObj = function (matchingText) {
        var month = monthStrToInt(matchingText);
        if (month === null)
            return null;
        var year = DateTime.utc().year;
        if (month < DateTime.utc().month) {
            year += 1;
        }
        if (contains(matchingText, MONTH.PAST_WORDS + " " + MONTH.ANY)) {
            year -= 1;
        }
        return DateTime.utc()
            .set({ month: month, year: year })
            .startOf('month')
            .toJSDate();
    };
    return Month;
}());

var YEAR = {
    ANY: '(\\b\\d{4}\\b)',
    FILLER_WORDS: 'in ',
};

var Year = (function () {
    function Year() {
    }
    Year.parseText = function (text) {
        var _this = this;
        var pattern = "(" + YEAR.FILLER_WORDS + ")?" + YEAR.ANY;
        var matches = matchPattern(text, pattern);
        if (!matches)
            return null;
        return matches.map(function (match) {
            var dateTimeObj = _this.convertMatchToDateObj(match);
            return parseMatches(text, match, dateTimeObj);
        });
    };
    Year.convertMatchToDateObj = function (matchingText) {
        var year = DateTime.utc().year;
        if (contains(matchingText, YEAR.ANY)) {
            var _a = __read(matchPattern(matchingText, YEAR.ANY), 1), matchedDay = _a[0];
            year = parseInt(matchedDay, 10);
        }
        if (!year)
            return null;
        return DateTime.utc()
            .set({ year: year })
            .startOf('year')
            .toJSDate();
    };
    return Year;
}());

var AM = 'am|a.m.|am.';
var PM = 'pm|p.m.|pm.';
var MERIDIEM = "(" + AM + "|" + PM + ")";
var FILLER_WORDS = '(at|by)';
var HOUR_PART = '([0-9]{1,2})';
var DIVIDER = '(:)';
var MINUTE_PART = '([0-9]{1,2})';
var TO = '(to|until)';
var PAST = '(after|past)';
var RELATIVE_TIME_FILLER_WORDS = "(" + TO + "|" + PAST + ")";
var HALF = 'half';
var QUARTER = 'quarter';
var VERBAL_QUANTIFIERS = "(" + HALF + "|" + QUARTER + ")";
var MINUTE_IDENTIFIER = '(min|mins|minutes)';
var FORMAT_JUST_HOUR = HOUR_PART + "( )?" + MERIDIEM;
var FORMAT_HOUR_WITH_MINS = "" + HOUR_PART + DIVIDER + MINUTE_PART;
var FORMAT_HOUR_WITH_MINS_AND_MERIDIEM = "" + HOUR_PART + DIVIDER + MINUTE_PART + "( )?" + MERIDIEM;
var FORMAT_VERBAL_QUANTIFIERS = VERBAL_QUANTIFIERS + "( )" + RELATIVE_TIME_FILLER_WORDS + "( )(" + HOUR_PART + "( )?" + MERIDIEM + "|" + HOUR_PART + ")";
var FORMAT_DIGIT_RELATIVE = MINUTE_PART + "( )(" + MINUTE_IDENTIFIER + "( ))?" + RELATIVE_TIME_FILLER_WORDS + "( )(" + HOUR_PART + "( )?" + MERIDIEM + "|" + HOUR_PART + ")";
var FORMAT_NORMAL_TIME = "(" + FORMAT_JUST_HOUR + "|" + FORMAT_HOUR_WITH_MINS_AND_MERIDIEM + "|" + FORMAT_HOUR_WITH_MINS + ")";
var FORMAT_RELATIVE_TIME = "(" + FORMAT_VERBAL_QUANTIFIERS + "|" + FORMAT_DIGIT_RELATIVE + ")";
var TIME = {
    ANY: "(" + FORMAT_NORMAL_TIME + "|" + FORMAT_RELATIVE_TIME + ")",
    FORMAT_NORMAL: FORMAT_NORMAL_TIME,
    FORMAT_RELATIVE: FORMAT_RELATIVE_TIME,
    RELATIVE_TIME_FILLER_WORDS: RELATIVE_TIME_FILLER_WORDS,
    HOUR_PART: HOUR_PART,
    MINUTE_PART: MINUTE_PART,
    DIVIDER: DIVIDER,
    MERIDIEM: MERIDIEM,
    AM: AM,
    PM: PM,
    TO: TO,
    PAST: PAST,
    HALF: HALF,
    QUARTER: QUARTER,
    VERBAL_QUANTIFIERS: VERBAL_QUANTIFIERS,
    MINUTE_IDENTIFIER: MINUTE_IDENTIFIER,
    FILLER_WORDS: FILLER_WORDS,
};

function getValidMatch(text, pattern) {
    var matched = matchPattern(text, pattern, false);
    if (!matched || matched.length === 0)
        return null;
    var _a = __read(matched, 1), value = _a[0];
    return value;
}
function convertTimeStringToObj(timeString) {
    var hour;
    var minute;
    var isRelativeTime = contains(timeString, TIME.RELATIVE_TIME_FILLER_WORDS);
    if (isRelativeTime) {
        if (contains(timeString, TIME.VERBAL_QUANTIFIERS)) {
            hour = getValidMatch(timeString, "(?<=" + TIME.RELATIVE_TIME_FILLER_WORDS + "( ))" + TIME.HOUR_PART);
            if (contains(timeString, TIME.HALF))
                minute = '30';
            if (contains(timeString, TIME.QUARTER))
                minute = '15';
        }
        else {
            hour = getValidMatch(timeString, "(?<=" + TIME.RELATIVE_TIME_FILLER_WORDS + "( ))" + TIME.HOUR_PART + "(?=((( )" + TIME.MERIDIEM + "))?)");
            minute = getValidMatch(timeString, "\\b" + TIME.MINUTE_PART + "(?=(( )(" + TIME.MINUTE_IDENTIFIER + "( ))?" + TIME.RELATIVE_TIME_FILLER_WORDS + "))");
        }
    }
    else {
        if (contains(timeString, TIME.DIVIDER)) {
            hour = getValidMatch(timeString, "\\b" + TIME.HOUR_PART + "(?=" + TIME.DIVIDER + ")");
            minute = getValidMatch(timeString, "(?<=" + TIME.DIVIDER + ")" + TIME.MINUTE_PART);
        }
        else {
            hour = getValidMatch(timeString, "\\b" + TIME.HOUR_PART + "(?=(( )?" + TIME.MERIDIEM + "))");
            minute = 0;
        }
    }
    hour = parseInt(hour, 10);
    minute = parseInt(minute, 10);
    if (isNaN(hour) || isNaN(minute) || hour > 23 || minute > 59)
        return null;
    if (contains(timeString, TIME.AM, false) && hour === 12)
        hour = 24;
    if (contains(timeString, TIME.TO) && hour > 0)
        hour -= 1;
    if (contains(timeString, TIME.TO))
        minute = 60 - minute;
    if (contains(timeString, TIME.PM, false) && hour < 12)
        hour += 12;
    if (hour === 24)
        hour = 0;
    return { hour: hour, minute: minute };
}

var Time = (function () {
    function Time() {
    }
    Time.parseText = function (text) {
        var _this = this;
        var pattern = "(" + TIME.FILLER_WORDS + "( ))?" + TIME.ANY;
        var matches = matchPattern(text, pattern, false);
        if (!matches)
            return null;
        return matches.map(function (match) {
            var dateTimeObj = _this.convertMatchToDateObj(match);
            return parseMatches(text, match, dateTimeObj);
        });
    };
    Time.convertMatchToDateObj = function (matchingText) {
        var timeObj = convertTimeStringToObj(matchingText);
        if (!timeObj)
            return null;
        var hour = timeObj.hour, minute = timeObj.minute;
        var newDateTime = DateTime.utc().set({ hour: hour, minute: minute });
        if (!newDateTime.isValid)
            return null;
        return newDateTime.startOf('minute').toJSDate();
    };
    return Time;
}());

var TODAY = 'today';
var TOMORROW = 'tomorrow';
var RELATIVE_ADVERB = {
    ANY: TODAY + "|" + TOMORROW,
    TODAY: "" + TODAY,
    TOMORROW: "" + TOMORROW,
};
var RELATIVE_PREPOSITIONS = '((in|after) )?';
var RELATIVE_POSTPOSITIONS = '( (from now|from today|later|after))';
var DAYS = '(days|day)';
var WEEKS = '(weeks|week|wks|wk)';
var MONTHS = '(months|month)';
var YEARS = '(years|year|yrs|yr)';
var TIME_UNITS = {
    DAYS: "" + DAYS,
    WEEKS: "" + WEEKS,
    MONTHS: "" + MONTHS,
    YEARS: "" + YEARS,
    ANY: "(" + DAYS + "|" + WEEKS + "|" + MONTHS + "|" + YEARS + ")",
};
var NEXT = '(next|following)';
var ONE = 'a';
var VERBAL_QUANTIFIERS$1 = {
    ANY: "(" + NEXT + "|" + ONE + ")",
    NEXT: "" + NEXT,
    ONE: "" + ONE,
};
var ARGUMENT = "([0-9]+|" + VERBAL_QUANTIFIERS$1.ANY + ") " + TIME_UNITS.ANY;
var ARGUMENT_AFTER = RELATIVE_PREPOSITIONS + "?" + ARGUMENT;
var ARGUMENT_FIRST = "" + ARGUMENT + RELATIVE_POSTPOSITIONS;
var RELATIVE_EXPRESSION = {
    ANY: "(" + ARGUMENT_FIRST + "|" + ARGUMENT_AFTER + ")",
    ARGUMENT_AFTER: "" + ARGUMENT_AFTER,
    ARGUMENT_FIRST: "" + ARGUMENT_FIRST,
};
var RELATIVE_DATES = {
    ANY: "(" + RELATIVE_ADVERB.ANY + "|" + RELATIVE_EXPRESSION.ANY + ")",
    RELATIVE_ADVERB: "" + RELATIVE_ADVERB.ANY,
    RELATIVE_EXPRESSION: "" + RELATIVE_EXPRESSION,
    FILLER_WORDS: RELATIVE_PREPOSITIONS + "|" + RELATIVE_POSTPOSITIONS,
    TIME_UNITS: TIME_UNITS,
    VERBAL_QUANTIFIERS: VERBAL_QUANTIFIERS$1,
};

var numDividers = '(\/|\-)';
var numDayMonthYear = "(" + DAY.NUMBERS + numDividers + MONTH.NUMBERS + numDividers + YEAR.ANY + ")";
var numMonthDayYear = "(" + MONTH.NUMBERS + numDividers + DAY.NUMBERS + numDividers + YEAR.ANY + ")";
var numYearMonthDay = "(" + YEAR.ANY + numDividers + MONTH.NUMBERS + numDividers + DAY.NUMBERS + ")";
var txtDividers = '((\,)? )';
var txtDayMonthYear = "(" + DAY.ANY + "?" + txtDividers + MONTH.ANY + txtDividers + YEAR.ANY + ")";
var txtMonthDayYear = "(" + MONTH.ANY + txtDividers + DAY.ANY + "?" + txtDividers + YEAR.ANY + ")";
var DATES = {
    ANY: numDayMonthYear + "|" + numMonthDayYear + "|" + numYearMonthDay + "|" + txtDayMonthYear + "|" + txtMonthDayYear,
    NUM_DAY_MONTH_YEAR: numDayMonthYear,
    NUM_MONTH_DAY_YEAR: numMonthDayYear,
    NUM_YEAR_MONTH_DAY: numYearMonthDay,
    TXT_DAY_MONTH_YEAR: txtDayMonthYear,
    TXT_MONTH_DAY_YEAR: txtMonthDayYear,
    FILLER_WORDS: '(on|by) (the )?',
    NUM_DIVIDER: numDividers,
    TXT_DIVIDER: txtDividers,
};
var numMonthYear = "(" + MONTH.NUMBERS + numDividers + YEAR.ANY + ")";
var numYearMonth = "(" + YEAR.ANY + numDividers + MONTH.NUMBERS + ")";
var numMonthDay = "(" + MONTH.NUMBERS + numDividers + DAY.NUMBERS + ")";
var numDayMonth = "(" + DAY.NUMBERS + numDividers + MONTH.NUMBERS + ")";
var txtMonthDay = "(" + MONTH.ANY + txtDividers + DAY.ANY + ")";
var txtDayMonth = "(" + DAY.ANY + txtDividers + MONTH.ANY + ")";
var txtMonthYear = "(" + MONTH.ANY + txtDividers + YEAR.ANY + ")";
var txtYearMonth = "(" + YEAR.ANY + txtDividers + MONTH.ANY + ")";
var day$1 = "" + DAY.ORDINAL_ONLY;
var PARTIAL_DATES = {
    ANY: "(" + numMonthYear + "|" + numYearMonth + "|" + numMonthDay + "|" + numDayMonth + "|" + txtMonthYear + "|" + txtMonthDay + "|" + txtYearMonth + "|" + txtDayMonth + "|" + day$1 + ")",
    NUM_MONTH_YEAR: numMonthYear,
    NUM_YEAR_MONTH: numYearMonth,
    NUM_MONTH_DAY: numMonthDay,
    NUM_DAY_MONTH: numDayMonth,
    TXT_MONTH_YEAR: txtMonthYear,
    TXT_MONTH_DAY: txtMonthDay,
    TXT_DAY_MONTH: txtDayMonth,
    TXT_YEAR_MONTH: txtYearMonth,
    DAY: day$1,
};
var datePart = "(" + DATES.FILLER_WORDS + ")?(" + DATES.ANY + "|" + PARTIAL_DATES.ANY + "|" + RELATIVE_DATES.ANY + ")";
var timePart = "(" + TIME.FILLER_WORDS + "( ))?" + TIME.ANY;
var dateFirst = datePart + "( )" + timePart;
var timeFirst = timePart + "( )" + datePart;
var DATE_AND_TIME = {
    ANY: dateFirst + "|" + timeFirst,
    DATE_FIRST: dateFirst,
    TIME_FIRST: timeFirst,
};

function strToInt(dayStr, monthStr, yearStr) {
    return {
        day: parseInt(dayStr, 10),
        month: parseInt(monthStr, 10),
        year: parseInt(yearStr, 10),
    };
}

function convertDateStringToObj(date) {
    var _a, _b, _c, _d, _e;
    var day;
    var month;
    var year;
    var dividerRegex;
    if (contains(date, DATES.NUM_DAY_MONTH_YEAR, false)) {
        dividerRegex = new RegExp(DATES.NUM_DIVIDER, 'g');
        _a = __read(date.replace(dividerRegex, ' ').split(' '), 3), day = _a[0], month = _a[1], year = _a[2];
    }
    else if (contains(date, DATES.NUM_MONTH_DAY_YEAR, false)) {
        dividerRegex = new RegExp(DATES.NUM_DIVIDER, 'g');
        _b = __read(date.replace(dividerRegex, ' ').split(' '), 3), month = _b[0], day = _b[1], year = _b[2];
    }
    else if (contains(date, DATES.NUM_YEAR_MONTH_DAY, false)) {
        dividerRegex = new RegExp(DATES.NUM_DIVIDER, 'g');
        _c = __read(date.replace(dividerRegex, ' ').split(' '), 3), year = _c[0], month = _c[1], day = _c[2];
    }
    else if (contains(date, DATES.TXT_DAY_MONTH_YEAR, false)) {
        dividerRegex = new RegExp(DATES.TXT_DIVIDER, 'gi');
        _d = __read(date.replace(dividerRegex, ' ').split(' '), 3), day = _d[0], month = _d[1], year = _d[2];
        month = monthStrToInt(month).toString();
    }
    else if (contains(date, DATES.TXT_MONTH_DAY_YEAR, false)) {
        dividerRegex = new RegExp(DATES.TXT_DIVIDER, 'gi');
        _e = __read(date.replace(dividerRegex, ' ').split(' '), 3), month = _e[0], day = _e[1], year = _e[2];
        month = monthStrToInt(month).toString();
    }
    if (!day || !month || !year)
        return null;
    return strToInt(day, month, year);
}

function remove(text, pattern, wordBoundary) {
    if (wordBoundary === void 0) { wordBoundary = true; }
    var updatedPattern = (wordBoundary) ? "\\b" + pattern + "\\b" : pattern;
    var regex = new RegExp(updatedPattern, 'ig');
    var replacedText = text.replace(regex, '');
    return trimWhiteSpaces(replacedText);
}

var Dates = (function () {
    function Dates() {
    }
    Dates.parseText = function (text) {
        var _this = this;
        var pattern = "(" + DATES.FILLER_WORDS + ")?(" + DATES.ANY + ")";
        var matches = matchPattern(text, pattern, false);
        if (!matches)
            return null;
        return matches.map(function (match) {
            var dateTimeObj = _this.convertMatchToDateObj(match);
            return parseMatches(text, match, dateTimeObj);
        });
    };
    Dates.convertMatchToDateObj = function (matchingText) {
        var removedFillerWords = remove(matchingText, DATES.FILLER_WORDS);
        var dateObj = convertDateStringToObj(removedFillerWords);
        if (!dateObj)
            return null;
        var day = dateObj.day, month = dateObj.month, year = dateObj.year;
        var newDateTime = DateTime.utc().set({ day: day, month: month, year: year });
        if (!newDateTime.isValid)
            return null;
        return newDateTime.endOf('day').toJSDate();
    };
    return Dates;
}());

function getFutureYearIfDateIsInThePast(monthStr, dayStr) {
    var currentDate = DateTime.utc();
    var year = currentDate.year;
    var month = parseInt(monthStr, 10);
    var day = parseInt(dayStr, 10);
    var tempDate = DateTime.utc().set({ month: month, day: day, year: year });
    if (tempDate < currentDate) {
        return (year + 1).toString();
    }
    return year.toString();
}
function getNextMonthIfDayIsInThePast(dayStr) {
    var currentDate = DateTime.utc();
    var day = parseInt(dayStr, 10);
    var currMonth = currentDate.month;
    var month = day < currentDate.day ? currMonth + 1 : currMonth;
    return month < 13 ? month : 1;
}
function convertPartialDateStringToObj(date) {
    var _a, _b, _c, _d, _e, _f, _g, _h;
    var day;
    var month;
    var year;
    var dividerRegex;
    if (contains(date, PARTIAL_DATES.NUM_MONTH_YEAR)) {
        dividerRegex = new RegExp(DATES.NUM_DIVIDER, 'g');
        _a = __read(date.replace(dividerRegex, ' ').split(' '), 2), month = _a[0], year = _a[1];
        day = '1';
    }
    else if (contains(date, PARTIAL_DATES.NUM_YEAR_MONTH)) {
        dividerRegex = new RegExp(DATES.NUM_DIVIDER, 'g');
        _b = __read(date.replace(dividerRegex, ' ').split(' '), 2), year = _b[0], month = _b[1];
        day = '1';
    }
    else if (contains(date, PARTIAL_DATES.NUM_DAY_MONTH)) {
        dividerRegex = new RegExp(DATES.NUM_DIVIDER, 'g');
        _c = __read(date.replace(dividerRegex, ' ').split(' '), 2), day = _c[0], month = _c[1];
        year = getFutureYearIfDateIsInThePast(month, day);
    }
    else if (contains(date, PARTIAL_DATES.NUM_MONTH_DAY)) {
        dividerRegex = new RegExp(DATES.NUM_DIVIDER, 'g');
        _d = __read(date.replace(dividerRegex, ' ').split(' '), 2), month = _d[0], day = _d[1];
        year = getFutureYearIfDateIsInThePast(month, day);
    }
    else if (contains(date, PARTIAL_DATES.TXT_MONTH_YEAR)) {
        dividerRegex = new RegExp(DATES.TXT_DIVIDER, 'g');
        _e = __read(date.replace(dividerRegex, ' ').split(' '), 2), month = _e[0], year = _e[1];
        day = '1';
        month = monthStrToInt(month).toString();
    }
    else if (contains(date, PARTIAL_DATES.TXT_YEAR_MONTH)) {
        dividerRegex = new RegExp(DATES.TXT_DIVIDER, 'g');
        _f = __read(date.replace(dividerRegex, ' ').split(' '), 2), year = _f[0], month = _f[1];
        day = '1';
        month = monthStrToInt(month).toString();
    }
    else if (contains(date, PARTIAL_DATES.TXT_DAY_MONTH)) {
        dividerRegex = new RegExp(DATES.TXT_DIVIDER, 'g');
        _g = __read(date.replace(dividerRegex, ' ').split(' '), 2), day = _g[0], month = _g[1];
        month = monthStrToInt(month).toString();
        year = getFutureYearIfDateIsInThePast(month, day);
    }
    else if (contains(date, PARTIAL_DATES.TXT_MONTH_DAY)) {
        dividerRegex = new RegExp(DATES.TXT_DIVIDER, 'g');
        _h = __read(date.replace(dividerRegex, ' ').split(' '), 2), month = _h[0], day = _h[1];
        month = monthStrToInt(month).toString();
        year = getFutureYearIfDateIsInThePast(month, day);
    }
    else if (contains(date, PARTIAL_DATES.DAY)) {
        day = date;
        month = getNextMonthIfDayIsInThePast(day).toString();
        year = getFutureYearIfDateIsInThePast(month, date);
    }
    if (!day || !month || !year)
        return null;
    return strToInt(day, month, year);
}

var PartialDates = (function () {
    function PartialDates() {
    }
    PartialDates.parseText = function (text) {
        var _this = this;
        var pattern = "(" + DATES.FILLER_WORDS + ")?(" + PARTIAL_DATES.ANY + ")";
        var matches = matchPattern(text, pattern, false);
        if (!matches)
            return null;
        return matches.map(function (match) {
            var dateTimeObj = _this.convertMatchToDateObj(match);
            return parseMatches(text, match, dateTimeObj);
        });
    };
    PartialDates.convertMatchToDateObj = function (matchingText) {
        var removedFillerWords = remove(matchingText, DATES.FILLER_WORDS);
        var dateObj = convertPartialDateStringToObj(removedFillerWords);
        if (!dateObj)
            return null;
        var day = dateObj.day, month = dateObj.month, year = dateObj.year;
        var newDateTime = DateTime.utc().set({ day: day, month: month, year: year });
        if (!newDateTime.isValid)
            return null;
        return newDateTime.endOf('day').toJSDate();
    };
    return PartialDates;
}());

function findMatchingKey(object, pattern) {
    var keys = Object.keys(object);
    if (keys.length) {
        for (var i = 0; i < keys.length; i++) {
            var key = keys[i];
            if (contains(object[key], pattern))
                return key;
        }
    }
    return null;
}

function timeUnitToString(matchAgainst) {
    return findMatchingKey(RELATIVE_DATES.TIME_UNITS, matchAgainst);
}
function convertRelativeAdverbToObj(relativeDateStr) {
    if (contains(relativeDateStr, RELATIVE_ADVERB.TODAY)) {
        return DateTime.utc();
    }
    return DateTime.utc().plus({ days: 1 });
}
function getNext(unit) {
    var _a;
    return DateTime.utc()
        .plus((_a = {}, _a[unit] = 1, _a))
        .startOf(unit)
        .endOf('day');
}
function convertRelativeExpressionToObj(expression) {
    var _a;
    var _b = __read(matchPattern(expression, RELATIVE_DATES.TIME_UNITS.ANY), 1), timeUnit = _b[0];
    var unit = timeUnitToString(timeUnit);
    var period = remove(expression, unit);
    var quantity;
    if (contains(period, RELATIVE_DATES.VERBAL_QUANTIFIERS.ONE)) {
        quantity = 1;
    }
    else if (contains(period, RELATIVE_DATES.VERBAL_QUANTIFIERS.NEXT)) {
        return getNext(unit);
    }
    else {
        quantity = parseInt(period, 10);
    }
    return DateTime.utc().plus((_a = {}, _a[unit] = quantity, _a));
}
function convertRelativeDateStringToObj(date) {
    var removedFillerWords = remove(date, RELATIVE_DATES.FILLER_WORDS);
    if (contains(removedFillerWords, RELATIVE_DATES.RELATIVE_ADVERB)) {
        return convertRelativeAdverbToObj(removedFillerWords);
    }
    else {
        return convertRelativeExpressionToObj(removedFillerWords);
    }
}

var DateAndTime = (function () {
    function DateAndTime() {
    }
    DateAndTime.parseText = function (text) {
        var _this = this;
        var pattern = "" + DATE_AND_TIME.ANY;
        var matches = matchPattern(text, pattern, false);
        if (!matches)
            return null;
        return matches.map(function (match) {
            var dateTimeObj = _this.convertMatchToDateObj(match);
            return parseMatches(text, match, dateTimeObj);
        });
    };
    DateAndTime.convertMatchToDateObj = function (matchingText) {
        var removeDateFillerWords = remove(matchingText, DATES.FILLER_WORDS);
        var dateStringMatches = matchPattern(removeDateFillerWords, "(" + DATES.ANY + "|" + PARTIAL_DATES.ANY + "|" + RELATIVE_DATES.ANY + ")");
        if (!dateStringMatches)
            return null;
        var dateString = dateStringMatches[0];
        var timeString = remove(removeDateFillerWords, dateString);
        var removedTimeFillerWords = remove(timeString, TIME.FILLER_WORDS);
        var dateObj;
        if (contains(dateString, "" + DATES.ANY)) {
            dateObj = convertDateStringToObj(dateString);
        }
        else if (contains(dateString, "" + PARTIAL_DATES.ANY)) {
            dateObj = convertPartialDateStringToObj(dateString);
        }
        else {
            dateObj = convertRelativeDateStringToObj(dateString);
        }
        if (!dateObj)
            return null;
        var day = dateObj.day, month = dateObj.month, year = dateObj.year;
        var timeObj = convertTimeStringToObj(removedTimeFillerWords);
        if (!timeObj)
            return null;
        var hour = timeObj.hour, minute = timeObj.minute;
        var newDateTime = DateTime.utc().set({ day: day, month: month, year: year, hour: hour, minute: minute });
        if (!newDateTime.isValid)
            return null;
        return newDateTime.startOf('minute').toJSDate();
    };
    return DateAndTime;
}());

var anyWeekday = "((" + WEEKDAY.FUTURE_WORDS + "|" + WEEKDAY.PAST_WORDS + ")( ))?" + WEEKDAY.ANY;
var anyTime = "(" + TIME.FILLER_WORDS + "( ))?" + TIME.FORMAT_NORMAL;
var timeFirst$1 = anyTime + "( )" + anyWeekday;
var weekdayFirst = anyWeekday + "( )" + anyTime;
var WEEKDAY_AND_TIME = {
    ANY: timeFirst$1 + "|" + weekdayFirst,
};

function getValidMatch$1(text, pattern) {
    var matched = matchPattern(text, pattern);
    if (!matched || matched.length === 0)
        return null;
    var _a = __read(matched, 1), value = _a[0];
    return value;
}
var WeekDayAndTime = (function () {
    function WeekDayAndTime() {
    }
    WeekDayAndTime.parseText = function (text) {
        var _this = this;
        var pattern = WEEKDAY_AND_TIME.ANY;
        var matchForWeekdayAndTime = matchPattern(text, pattern);
        if (!matchForWeekdayAndTime)
            return null;
        return matchForWeekdayAndTime.map(function (elem) { return _this.parseWeekdayAndTimeMatches(text, elem); });
    };
    WeekDayAndTime.parseWeekdayAndTimeMatches = function (text, matchedWeekday) {
        var replaceMatch = text.toLowerCase().replace(matchedWeekday, '');
        return {
            text: trimWhiteSpaces(replaceMatch),
            dateTime: this.convertWeekdayAndTimeToDateObj(matchedWeekday),
            matched: trimWhiteSpaces(matchedWeekday),
        };
    };
    WeekDayAndTime.convertWeekdayAndTimeToDateObj = function (matchingText) {
        var timeString = getValidMatch$1(matchingText, TIME.ANY);
        var _a = convertTimeStringToObj(timeString), hour = _a.hour, minute = _a.minute;
        var _b = __read(matchPattern(matchingText, WEEKDAY.ANY), 1), weekdayString = _b[0];
        var pastWeekday = contains(matchingText, WEEKDAY.PAST_WORDS);
        var weekday = convertWeekdayStringToNumber(weekdayString, pastWeekday);
        var newDateTime = DateTime.utc().set({ weekday: weekday, hour: hour, minute: minute });
        if (!newDateTime.isValid)
            return null;
        return newDateTime.startOf('minute').toJSDate();
    };
    return WeekDayAndTime;
}());

var RELATIVE_PREPOSITIONS$1 = '(in|after) ';
var RELATIVE_POSTPOSTIONS = ' (from now|later|after)';
var HOURS = '(hours|hour|hrs|hr)';
var MINUTES = '(minutes|minute|mins|min)';
var SECONDS = '(seconds|second|secs|sec)';
var TIME_UNITS$1 = {
    ANY: "(" + HOURS + "|" + MINUTES + "|" + SECONDS + ")",
    HOURS: "" + HOURS,
    MINUTES: "" + MINUTES,
    SECONDS: "" + SECONDS,
};
var HALF$1 = '(half an|half a)';
var QUARTER$1 = '(a quarter of a|a quarter of an)';
var ONE$1 = '(a|an|one)';
var VERBAL_QUANTIFIERS$2 = {
    ANY: "(" + HALF$1 + "|" + QUARTER$1 + "|" + ONE$1 + ")",
    HALF: "" + HALF$1,
    QUARTER: "" + QUARTER$1,
    ONE: "" + ONE$1,
};
var ARGUMENT$1 = "([0-9]+|" + VERBAL_QUANTIFIERS$2.ANY + ") " + TIME_UNITS$1.ANY;
var ARGUMENT_AFTER$1 = "" + RELATIVE_PREPOSITIONS$1 + ARGUMENT$1;
var ARGUMENT_FIRST$1 = "" + ARGUMENT$1 + RELATIVE_POSTPOSTIONS;
var RELATIVE_TIME = {
    ANY: "(" + ARGUMENT_FIRST$1 + "|" + ARGUMENT_AFTER$1 + ")",
    ARGUMENT_AFTER: "" + ARGUMENT_AFTER$1,
    ARGUMENT_FIRST: "" + ARGUMENT_FIRST$1,
    FILLER_WORDS: RELATIVE_PREPOSITIONS$1 + "|" + RELATIVE_POSTPOSTIONS,
    TIME_UNITS: TIME_UNITS$1,
    VERBAL_QUANTIFIERS: VERBAL_QUANTIFIERS$2,
};

var RelativeTime = (function () {
    function RelativeTime() {
    }
    RelativeTime.parseText = function (text) {
        var _this = this;
        var pattern = RELATIVE_TIME.ANY;
        var matches = matchPattern(text, pattern, false);
        if (!matches)
            return null;
        return matches.map(function (match) {
            var dateTimeObj = _this.convertMatchToDateObj(match);
            return parseMatches(text, match, dateTimeObj);
        });
    };
    RelativeTime.convertRelativeTimeStringToNumericValue = function (timePeriod, timeUnit) {
        var timeValue = 1000;
        if (contains(timePeriod, RELATIVE_TIME.VERBAL_QUANTIFIERS.QUARTER)) {
            timeValue *= 0.25;
        }
        else if (contains(timePeriod, RELATIVE_TIME.VERBAL_QUANTIFIERS.HALF)) {
            timeValue *= 0.5;
        }
        else if (contains(timePeriod, RELATIVE_TIME.VERBAL_QUANTIFIERS.ONE)) {
            timeValue *= 1;
        }
        else {
            timeValue *= parseInt(timePeriod, 10);
        }
        if (contains(timeUnit, RELATIVE_TIME.TIME_UNITS.HOURS)) {
            timeValue *= 3600;
        }
        else if (contains(timeUnit, RELATIVE_TIME.TIME_UNITS.MINUTES)) {
            timeValue *= 60;
        }
        return timeValue;
    };
    RelativeTime.addRelativeTimeToCurrentTime = function (timeString) {
        var timeUnit = matchPattern(timeString, RELATIVE_TIME.TIME_UNITS.ANY)[0];
        var timePeriod = remove(timeString, timeUnit);
        var timeValue = this.convertRelativeTimeStringToNumericValue(timePeriod, timeUnit);
        return DateTime.utc().plus(timeValue);
    };
    RelativeTime.convertMatchToDateObj = function (matchingText) {
        var removedFillerWords = remove(matchingText, RELATIVE_TIME.FILLER_WORDS);
        var newDateTime = this.addRelativeTimeToCurrentTime(removedFillerWords);
        if (!newDateTime.isValid)
            return null;
        return newDateTime.startOf('second').toJSDate();
    };
    return RelativeTime;
}());

var RelativeDates = (function () {
    function RelativeDates() {
    }
    RelativeDates.parseText = function (text) {
        var _this = this;
        var pattern = "(" + DATES.FILLER_WORDS + ")?" + RELATIVE_DATES.ANY;
        var matches = matchPattern(text, pattern, false);
        if (!matches)
            return null;
        return matches.map(function (match) {
            var dateTimeObj = _this.convertMatchToDateObj(match);
            return parseMatches(text, match, dateTimeObj);
        });
    };
    RelativeDates.convertMatchToDateObj = function (matchingText) {
        var dateObj = convertRelativeDateStringToObj(matchingText);
        if (!dateObj)
            return null;
        return dateObj.endOf('day').toJSDate();
    };
    return RelativeDates;
}());

var MID_NIGHT = '(midnight|mid night|mid-night)';
var MID_DAY = '(noon|midday|mid day|mid-day)';
var TIME_KEYWORDS = {
    ANY: "(" + MID_NIGHT + "|" + MID_DAY + ")",
    FILLER_WORDS: '(at|by|around)',
    MID_NIGHT: MID_NIGHT,
    MID_DAY: MID_DAY,
};

var TimeKeywords = (function () {
    function TimeKeywords() {
    }
    TimeKeywords.parseText = function (text) {
        var _this = this;
        var pattern = "(" + TIME_KEYWORDS.FILLER_WORDS + "( ))?" + TIME_KEYWORDS.ANY;
        var matches = matchPattern(text, pattern, false);
        if (!matches)
            return null;
        return matches.map(function (match) {
            var dateTimeObj = _this.convertMatchToDateObj(match);
            return parseMatches(text, match, dateTimeObj);
        });
    };
    TimeKeywords.convertMatchToDateObj = function (matchingText) {
        var day = DateTime.utc().day;
        var hour = null;
        var currentHour = DateTime.utc().hour;
        if (contains(matchingText, "" + TIME_KEYWORDS.MID_DAY)) {
            hour = 12;
            if (currentHour > 12)
                day += 1;
        }
        if (contains(matchingText, "" + TIME_KEYWORDS.MID_NIGHT)) {
            hour = 0;
            day += 1;
        }
        if (hour === null)
            return null;
        var newDateTime = DateTime.utc().set({ day: day, hour: hour, minute: 0 });
        if (!newDateTime.isValid)
            return null;
        return newDateTime.startOf('minute').toJSDate();
    };
    return TimeKeywords;
}());

var serina = function (text) {
    var parsedData = {
        original: text,
        isValid: false,
        matches: [],
    };
    var filters = [WeekDay, Day, Month, Year, Time, Dates, PartialDates, DateAndTime, RelativeTime, RelativeDates, WeekDayAndTime, TimeKeywords];
    filters.forEach(function (filter) {
        var results = filter.parseText(text);
        if (results && results.length)
            parsedData.matches = parsedData.matches.concat(results);
    });
    if (parsedData.matches.length)
        parsedData.isValid = true;
    return parsedData;
};

export default serina;
